@@ -1,5 +1,8 @@
 /* Bitset vectors.
-   Copyright (C) 2002, 2004, 2009-2010 Free Software Foundation, Inc.
+
+   Copyright (C) 2002, 2004, 2009-2015, 2018 Free Software Foundation,
+   Inc.
+
    Contributed by Akim Demaille (akim@freefriends.org).
 
    This program is free software: you can redistribute it and/or modify
@@ -21,6 +24,6 @@
 #include "bitsetv.h"
 
 /* Dump vector of bitsets as a matrix.  */
-extern void bitsetv_matrix_dump (FILE *, const char *, bitsetv);
+void bitsetv_matrix_dump (FILE *, const char *, bitsetv);
 
 #endif  /* _BITSETV_H  */
